

/***********************************************************

 Torre di Hanoi: un numero n di dischi, di diametro via
 via decrescente, sono impilati sul primo dei tre pioli della
 Torre di Hanoi. L'obiettivo  spostare tutti i dischi
 sull'ultimo piolo, rispettando le seguenti regole:
 - si puo` spostare un disco solo per volta, su un qualsiasi 
   piolo;
 - non si puo` impilare un disco su uno piu` piccolo.

 La soluzione ricorsiva si basa sulla seguente osservazione: 
 per spostare k dischi dal piolo 1 al piolo 2 bisogna prima 
 spostare i k-1 dischi superiori dal piolo 1 al piolo 3.

***********************************************************/

#include <stdio.h>
#include <stdlib.h>

#define MAXDISCHI 10
#define RMAXPIOLO 10

void hanoi (int n, int from, int to, int store);
void move (int from, int to);
void inizializza_pioli (int n);
void aggiorna_pioli(int from, int to);
void disegna_torre(void);
void disegna_disco(int raggio);

struct piolo
{
   int disco[MAXDISCHI];
   int ndischi;
};

struct piolo torre[3];


int main ()
{
   int numerodischi;

   do
   {
      printf ("Numero di dischi sul piolo di partenza? : ");
      scanf ("%d", &numerodischi);
      if (numerodischi > 10)
         printf ("massimo numero di dischi = %d\n", MAXDISCHI);
   } while (numerodischi > 10);
   while (getchar() != '\n');

   inizializza_pioli (numerodischi);
   hanoi (numerodischi, 0, 2, 1);

   return EXIT_SUCCESS;
}


void move (int from, int to)
{
   /* segnala lo spostamento di un disco */
   printf ("%d --> %d\n", from, to);
   aggiorna_pioli(from, to);
   disegna_torre();
   return;
}


void hanoi (int n, int from, int to, int store)
{
   if (n == 1)
      move (from, to);
   else
   {
      /* sposta la parte superiore della torre nel 
         piolo intermedio (store) */
      hanoi (n-1, from, store, to);
      /* ora sposta l'ultimo disco */
      move (from, to);
      /* e infine sposta la parte superiore della torre 
         dal piolo intermedio a quello di destinazione */
      hanoi (n-1, store, to, from);
   }
   return;
}

void inizializza_pioli (int n)
{
   int i, k;

   torre[0].ndischi = n;
   for (i = 0; i < n; i++)
      torre[0].disco[i] = n - i;

   for ( ; i < MAXDISCHI; i++)
      torre[0].disco[i] = 0;
      
   torre[2].ndischi = torre[3].ndischi = 0;
   for (i = 0; i < MAXDISCHI; i++)
      torre[1].disco[i] = torre[2].disco[i] = 0;

   return;
}

void aggiorna_pioli(int from, int to)
{
   int i;

   torre[to].disco[torre[to].ndischi] =
      torre[from].disco[torre[from].ndischi-1];

   torre[from].disco[torre[from].ndischi-1] = 0;
   torre[from].ndischi--;

   torre[to].ndischi++;

   return;
}


void disegna_torre(void)
{
   int piolo, altezza, i;

   printf ("\n");
   for (altezza = MAXDISCHI-1; altezza >= 0; altezza--)
   {
      for (piolo = 0; piolo < 3; piolo++)
      {
         disegna_disco (torre[piolo].disco[altezza]);
         printf (" ");
      }
      printf ("\n");
   }

   for (i = 0; i < 79; i++)
      printf ("_");
   printf ("\n");
   printf ("premere INVIO per continuare ");
   getchar();

   return;
}


void disegna_disco(int raggio)
{
   int i;

   for (i = 0; i < RMAXPIOLO - raggio; i++)
      printf (" ");

   for (i = 0; i < raggio; i++)
      printf ("*");

   printf ("|");

   for (i = 0; i < raggio; i++)
      printf ("*");

   for (i = 0; i < RMAXPIOLO - raggio; i++)
      printf (" ");

   return;
}
