# Calcolo della soluzione di un sistema lineare di n equazioni in n incognite.
# La funzione gauss() accetta la matrice completa che rappresenta il sistema 
# lineare da risolvere e si suppone che il sistema sia determinato
#
# Per esercizio, si puo' modificare il programma in modo che esso sia 
# robusto rispetto a una matrice di ingresso che rappresenta un sistema
# impossibile o indeterminato.
#

def stampa_matrice(m):
	"""Stampa la matrice arrotondando il valore degli elementi in modo 'generale', 
   	ovvero in virgola fissa oppure in formato scientifico (esponenziale)"""

	for i in range(0, len(m)):
		for j in range(0, len(m[i])):
			print('%g'% (m[i][j]), end="\t")
			
		print("")
		

def scambiariga(m, l):
	"""La funzione prende in ingresso una matrice m e un intero l che indica la riga 
   	al di sotto della quale viene cercato un elemento non nullo sulla colonna l.
   	Se lo trova, scambia la riga l con la prima riga trovata con elemento non nullo
   	sulla colonna l"""

	rows = len(m)
	cols = len(m[0])
	
	riga_scambiata = False
	for i in range(1, rows - l):

		# cerco l'elemento sotto a quello in posizione (l,l) finche' ne trovo uno 
		# non nullo
		#
        	if m[l+i][l] !=0:
        		# scambio la riga l con la l + i
        		for j in range(0, cols):
        			tmp = m[l + i][j]
        			m[l + i][j] = m[l][j]
        			m[l][j] = tmp
        			
        		riga_scambiata = True
        		# ok esco dal ciclo
        		break

	return riga_scambiata


def diagonale_unitaria(m):
	"""Questa funzione prende in ingresso una matrice e divide ogni elemento 
   	di una riga m[i][j] per l'elemento m[i][i], in modo che ogni elemento
   	m[i][i] sia pari ad 1."""

	rows = len(m)
	cols = len(m[0])
		
	for i in range(0, rows):
		mii = m[i][i]
		if mii != 0:
		# altrimenti riga nulla e sistema cala di rango...
			for j in range(0, cols):
				m[i][j] = m[i][j] / mii
		
		
def gauss(m):
	"""Questa funzione applica il metodo di eliminazione di Gauss per 
	   risolvere un sistema di equazioni lineari.
	   Ipotesi: il sistema rappresentato dalla matrice completa in 
	   ingresso deve essere determinato"""

	# controllo che la matrice non sia nulla
	if len(m) == 0 or len(m[0]) == 0:
		return "Matrice in input nulla!"
	
	# suppongo matrice valida, ove ogni riga ha lo stesso numero di elementi
	nrows = len(m)
	ncols = len(m[0]) 
	
	# suppongo sistema lineare n eq, n incognite.
	
	count = min(nrows, ncols)
	
	for row in range(0, count):
		
		m_row_row_nonzero = 1
		
		# elemento della diagonale nullo?
		if m[row][row] == 0: 
			# cerco sotto la colonna row-ma una riga con colonna row-ma non nulla
			m_row_row_nonzero = scambiariga(m, row)
	
		if m_row_row_nonzero:
			# devo annullare tutti gli elementi delle righe sotto su quella colonna
			next_row = row + 1
			
			while next_row < nrows:
				# calcolo il valore del coefficiente atto ad annullare 
				# l'elemento sotto quello di posto (row, row)
				#
				coeff = -m[next_row][row]/m[row][row];
				#
				# aggiungo agli elementi della riga next_row gli elementi 
				# della riga row moltiplicati 
				# per il coefficiente coeff calcolato. 
				# Il risultato e` che l'elemento sotto
				# a[row][row] viene annullato.
				#
				for column in range(0, ncols):
					m[next_row][column] = m[next_row][column] + coeff * m[row][column];
			
				# prossimo passo, applico il procedimento alla riga successiva
				next_row = next_row + 1
			#
			# ora serve annullare tutti gli elementi sopra la riga row nella colonna row
			prev_row = row - 1
			
			while prev_row >= 0:
				coeff = -m[prev_row][row] / m[row][row];
				for column in range(0, ncols):
					m[prev_row][column] = m[prev_row][column] + coeff * m[row][column];
				
				prev_row = prev_row - 1
	

	diagonale_unitaria(m)

	stampa_matrice(m)

# 	se voglio restituire la matrice al chiamante
#	
#	return m
	
	return 0
	
			
		



	
